#include "p30f6014A.h"

#define IR_RECIEVER

#include "stdio.h"
#include "string.h"
#include <time.h>

#include "./codec/e_sound.h"
#include "./motor_led/e_init_port.h"
#include "./motor_led/advance_one_timer/e_led.h"
#include "./motor_led/advance_one_timer/e_motors.h"
#include <motor_led/advance_one_timer/e_agenda.h>
#include <camera/fast_2_timer/e_poxxxx.h>
#include "./uart/e_uart_char.h"
//#include <uart/e_uart_char.h>
#include "./a_d/advance_ad_scan/e_ad_conv.h"
#include "./a_d/advance_ad_scan/e_prox.h"
#include <a_d/advance_ad_scan/e_acc.h>
#include <a_d/advance_ad_scan/e_micro.h>
#include "math.h"
#include "./bluetooth/e_bluetooth.h"
#include "./motor_led/e_epuck_ports.h"
/* selector on normal extension*/

#define SELECTOR0 _RG6

#define SELECTOR1 _RG7

#define SELECTOR2 _RG8

#define SELECTOR3 _RG9

#include "memory.h"
char buffer[BUFFER_SIZE]={'\0'};

#ifdef IR_RECIEVER
#include <motor_led/advance_one_timer/e_remote_control.h>
#define SPEED_IR 600
#endif 
#include "runaccelerometer.h"
#include "runbreitenberg_adv.h"
#include "runlocatesound.h"
#include "runwallfollow.h"
#include "rungrounddirection.h"
#include "utility.h"
#include "Asercom.h"
#include "runshow.h"
#include "translatorI2C.h"

#define PI 3.14159265358979

#define uart2_send_static_text(msg) { e_send_uart2_char(msg,sizeof(msg)-1); while(e_uart2_sending()); }
#define uart1_send_static_text(msg) { e_send_uart1_char(msg,sizeof(msg)-1); while(e_uart1_sending()); }

volatile static unsigned long timer_ticks=0;

void __attribute__((interrupt, auto_psv))

 _T1Interrupt(void)

{
	IFS0bits.T1IF = 0;
	TMR1 = 0;
	timer_ticks++;
	return;

}

int main() {
	//SLEEP(); while(1);

	int selector, version, i,j, r=-1, g=-1, b=-1;
	char c;
	//char command[20];
	//char response[50];
	int c0, c1; /* clock_t defined in <time.h> and <sys/types.h> as int */
	int t0, t1;
	char elapsed_time[40];
	char high_left, high_right, low_left, low_right;
	unsigned int bytes_sent=0, series=0, times=0;
	double timing;

	//system initialization 
	e_init_port();    // configure port pins
	e_start_agendas_processing();
	e_init_motors();
	//e_init_uart1(BAUD115200);   // initialize UART to 115200 Kbaud
	//e_init_uart2(BAUD115200);   // initialize UART to 115200 Kbaud
	e_init_ad_scan(ALL_ADC);
	//e_init_ad_scan();

	//Reset if Power on (some problem for few robots)
	if (RCONbits.POR) {
		RCONbits.POR=0;
		__asm__ volatile ("reset");
	}

	// Decide upon program
	selector=getselector();
	//sprintf(buffer, "Selector pos %d\r\n", selector);
	//e_send_uart1_char(buffer, strlen(buffer));
	//while(e_uart1_sending());


	if (selector==0) {
		run_accelerometer();
	} else if (selector==1) {
		run_locatesound();
	} else if (selector==2) {
		run_wallfollow();
	} else if (selector==3) {
		e_bt_write_operation_mode(0);
		run_asercom();		// advanced sercom protocol
	} else if (selector==4) {	// sensor "feedback display"
		e_init_uart1(BAUD115200);
		e_init_uart2(BAUD115200);
		do {
			c=e_bt_set_scanmode(0,0);
		} while(c!=0);
		//e_bt_write_operation_mode(0);		//attenzione perchè poi nn si può più programmare il robot
		//run_asercom();
		//e_bt_exit_transparent_mode();	//now the robot can accept commands
		while(1) {
			e_bt_recv_SPP_data(buffer);
			memset(elapsed_time, 0x0, 40);
			for(i=0; i<PACKET_SIZE; i++) {
				sprintf(&elapsed_time[i*3], "%02X:", buffer[i]);
			}
			//sprintf(elapsed_time, "%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x:%02x\r\n", buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5], buffer[6], buffer[7], buffer[8], buffer[9]);
			sprintf(&elapsed_time[i*3], "\r\n");
			e_send_uart2_char(elapsed_time, strlen(elapsed_time));
			while(e_uart2_sending());
			memset(buffer, 0x0, 50);
		}	
	} else if (selector==5) {	// sensor "feedback display"
		run_SensDispl();
	} else if (selector==6) {	// camera point to light
		run_CameraTurn();
	} else if (selector==7) {	// act like the ASL RS232 - I2C translator
		run_translatorI2C();
	} else if (selector==8) {	
		run_grounddirection();
	} else if (selector==9) {	// act as a Master BT and send data to a robot

		//T1CON = 0x8030; // Timer1 settings: TMR1 on (Timer Mode), prescale 1:256
		TMR1 = 0;			// clear timer 1

		T1CON = 0;			// reset Timer1 CONtrol register

		T1CONbits.TCKPS = 3;		// precsaler = 256
		//T1CONbits.TSYNC = 1;
		//PR1 = (int)(0.1*MILLISEC);	// interrupt every 0.1 ms with 64 prescaler

		PR1 = 0xFFFF;
		IFS0bits.T1IF = 0;		// clear interrupt flag


		e_init_uart1(BAUD115200);	//can be done only once!!
		e_init_uart2(BAUD115200);	//can be done only once!!
		e_bt_exit_transparent_mode();	//isn't necessary
		
		/*
		g=0;
		while(!g) {
			// set body led
			e_set_body_led(0);
			// set front led
			e_set_front_led(0);
			// set others LEDs
			e_set_led(8,0);
			i=e_bt_find_epuck();
			switch(i) {
				case 0: 
					// set body led
					e_set_body_led(1);
					// set front led
					e_set_front_led(0);
					// set others LEDs
					e_set_led(8,0);
					break;
				case 1: e_set_led(0,1);
					g=1;	//found one robot
					break;
				case 2: e_set_led(0,1);
					e_set_led(1,1);
					g=1;	//found two robots
					break;
				default: e_set_front_led(1);
			}
			for(r=0;r<30000;r++);
		}
		*/
		do {
			c=e_bt_connect_epuck();
		} while(c!=0);

/*
		//clear output buffer
    		memset(buffer, 0x0, 20);
    		sprintf(buffer, "\r");
		e_send_uart1_char(buffer, 1);
		while(e_uart1_sending());

    		memset(buffer, 0x0, 20);
    		sprintf(buffer, "V\r");
		e_send_uart1_char(buffer, 2);
		while(e_uart1_sending());

		//exit transparent mode
    		memset(buffer, 0x0, 20);
		buffer[0] = -'T';
		buffer[1] = 0;	
    		//sprintf(buffer, "%c%c", -'T', 0);	
		e_send_uart1_char(buffer, 2);
		while(e_uart1_sending());
		e_send_uart1_char(buffer, 2);
		while(e_uart1_sending());
*/
		//active to park mode
		/*do {
			c=e_bt_set_link_policy(0);
		} while(c!=0);
		c=e_bt_get_link_policy(0);
		sprintf(elapsed_time, "Poilicy=%d\r\n", c);	
		e_send_uart2_char(elapsed_time, strlen(elapsed_time));
		while(e_uart2_sending());
		timer_ticks=0;
		IEC0bits.T1IE = 1;		// set interrupt enable bit
		T1CONbits.TON = 1;		// start Timer1
		
		do {
			c=e_bt_enter_parkmode(0);
			sprintf(elapsed_time, "Park mode error=%d\r\n", c);	
			e_send_uart2_char(elapsed_time, strlen(elapsed_time));
			while(e_uart2_sending());
		} while(c!=0);
		
		//e_bt_enter_parkmode(0);
		T1CONbits.TON = 0;		// stop Timer1
		IEC0bits.T1IE = 0;		// disable timer1 interrupt
		timing = (double)(1.0/(FCY/256.0))*(double)TMR1 + (double)(1.0/(FCY/256.0))*(double)PR1*(double)timer_ticks;
		sprintf(elapsed_time, "Active->park delay=%f s\r\n", timing);	
		e_send_uart2_char(elapsed_time, strlen(elapsed_time));
		while(e_uart2_sending());
		*/
		/*
		do {
			c=e_bt_transparent_mode();
		} while(c!=0);
		*/

		//for(i=0; i<30000; i++) {
		//	uart2_send_static_text("ciao\n");
		//}
		
		/*
		while(1) {
			sprintf(buffer,"L,%d,%d\n\r",1,(i%2));
			if(i++ > 1000) {
				i=1;
			}
			//e_bt_send_SPP_data(buffer,strlen(buffer));
			e_send_uart1_char(buffer, strlen(buffer));
			while(e_uart1_sending());
			for(r=0;r<30000;r++);
		}
		*/

		//clear output buffer
		/*
    		memset(buffer, 0x0, 20);
    		sprintf(buffer, "\r");
		e_send_uart1_char(buffer, 1);
		while(e_uart1_sending());

    		memset(buffer, 0x0, 20);
    		sprintf(buffer, "V\r");
		e_send_uart1_char(buffer, 2);
		while(e_uart1_sending());
		*/

		/*
		do {
			c=e_bt_set_link_policy(0);
		} while(c!=0);
		do {
			c=e_bt_enter_parkmode(0);
			sprintf(elapsed_time, "Park mode error=%d\r\n", c);	
			e_send_uart2_char(elapsed_time, strlen(elapsed_time));
			while(e_uart2_sending());
		} while(c!=0);
		*/

		/*
		e_bt_enter_parkmode(0);
		for(r=0; r<10000; r++);
		g=0;
		b=0;
		while(1) {
			buffer[0]='c';
			buffer[1]='i';
			buffer[2]='a';
			buffer[3]='o';
			e_bt_send_SPP_data(buffer, 4);
			//wait a little
			for(r=0; r<3000; r++);
			//break;
			
			if(g++>100) {
				g=0;
				do {
					c=e_bt_enter_parkmode(0);
					sprintf(elapsed_time, "Park mode error=%d\r\n", c);	
					e_send_uart2_char(elapsed_time, strlen(elapsed_time));
					while(e_uart2_sending());
				} while(c!=0);
			}
			if(b++>200) {
				b=0;
				e_bt_exit_parkmode(0);
			}
			
		}
		*/

		for(i=0; i<PACKET_SIZE; i++) {
			buffer[i]=i;
		}

		while(1) {
			//if(times>=1)
			//	break;
			//times++;
			//timer_ticks=0;
			bytes_sent=0;
			timing=0.0;
			//t0=time(NULL);
			//IEC0bits.T1IE = 1;		// set interrupt enable bit
			//T1CONbits.TON = 1;		// start Timer1
			//TMR1=0;
			for(i=BUFFER_SIZE*10/PACKET_SIZE-1; i>=0; i--) {	//BUFFER_SIZE / 6 bytes per command = 4050 / 6 = 675 cycles
				high_left = (i>>8) & 0xFF;
				low_left = i & 0xFF;
				high_right = (i>>8) & 0xFF;
				low_right = i & 0xFF;
				//sprintf(&buffer[i*6], "%c%c%c%c%c%c",-'D', low_left, high_left, low_right, high_right,0);	//doesn't work => -'D' becomes zero...??
				/*
				buffer[i*6]=-'D';
				buffer[i*6+1]= low_left;
				buffer[i*6+2]= high_left;
				buffer[i*6+3]= low_right;
				buffer[i*6+4]= high_right;
				buffer[i*6+5]= 0;
				*/
	
				/*
				sprintf(elapsed_time, "%.2x%.2x%.2x%.2x%.2x%.2x\n", (unsigned char)buffer[i*6], (unsigned char)buffer[i*6+1], (unsigned char)buffer[i*6+2], (unsigned char)buffer[i*6+3], (unsigned char)buffer[i*6+4], (unsigned char)buffer[i*6+5]);
				e_send_uart2_char(elapsed_time, strlen(elapsed_time));
				while(e_uart2_sending());
				*/
				/*
				for(j=0; j<PACKET_SIZE; j+=CMD_SIZE) {
					buffer[j]=-'D';
					buffer[j+1]= low_left;
					buffer[j+2]= high_left;
					buffer[j+3]= low_right;
					buffer[j+4]= high_right;
					buffer[j+5]= 0;
				}*/			
				
				//send data to turn on the leds of the receiving robot sequentially
				/*
				//sprintf(buffer,"L,%d,%d\n\r",(i%8),(b%2));			//ascii mode
				sprintf(buffer,"%c%c%c%c",-'L', (i%8), (b%2),0);	//binary mode
				//%c imposta il valore di 8 bits corrispondente a -'L' nel byte del buffer di destinazione
				//%d imposta il valore intero rappresentato come carattere nel byte del buffer di destinazione
				//per esempio se l'ultimo byte metto %c, il quarto byte del buffer conterra' 0x00, mentre con %d conterra' 0x30
				if((i%8)==0)
					b++;
				*/
		
				//send commands to other robot through BT
				TMR1 = 0;	//reset timer
				timer_ticks = 0;
				IEC0bits.T1IE = 1;		// set interrupt enable bit
				T1CONbits.TON = 1;		// start Timer1
				e_bt_send_SPP_data(buffer, PACKET_SIZE);
				//e_send_uart1_char(buffer, PACKET_SIZE);
				//while(e_uart1_sending());;
				T1CONbits.TON = 0;		// stop Timer1
				IEC0bits.T1IE = 0;		// disable timer1 interrupt
				timing += (double)(1.0/(FCY/256.0))*(double)TMR1 + (double)(1.0/(FCY/256.0))*(double)PR1*(double)timer_ticks;
				bytes_sent += PACKET_SIZE;

				//send some information about commands to serial line	
				//memset(elapsed_time, 0x0, 40);
				//sprintf(elapsed_time,"[%d] bytes=%d\r\n", i, bytes_sent);
				//e_send_uart2_char(elapsed_time, strlen(elapsed_time));
				//while(e_uart2_sending());
	
				//wait a little
				//for(r=0; r<3000; r++);
				
			}
	
			//t1=time(NULL);
			//T1CONbits.TON = 0;		// stop Timer1
			//IEC0bits.T1IE = 0;		// disable timer1 interrupt
			
			//send binary command terminator
			/*
			buffer[0]=-'D';
			buffer[1]= low_left;
			buffer[2]= high_left;
			buffer[3]= low_right;
			buffer[4]= high_right;
			buffer[5]= 0;
			e_send_uart1_char(buffer, PACKET_SIZE+1);
			while(e_uart1_sending());
			for(r=0; r<3000; r++);		
			*/
	
			//send led command as the packet termination identifier
			//memset(buffer, 0x0, 6);
			//sprintf(buffer,"%c%c%c%c",-'L', 0, 1, 0);	//binary mode
			//e_send_uart1_char(buffer, 4);
			//while(e_uart1_sending());
	
			//c0 = clock();
			//t0 = time(NULL);
			//T1CONbits.TON = 1;		// start Timer1
			/*
			for(i=749; i>=0; i--) {	//BUFFER_SIZE / 6 bytes per command = 750
				e_send_uart1_char(&buffer[i*6], 6);
				while(e_uart1_sending());
				/*
				sprintf(elapsed_time, "%.2x%.2x%.2x%.2x%.2x%.2x\n", (unsigned char)buffer[i*6], (unsigned char)buffer[i*6+1], (unsigned char)buffer[i*6+2], (unsigned char)buffer[i*6+3], (unsigned char)buffer[i*6+4], (unsigned char)buffer[i*6+5]);
				e_send_uart2_char(elapsed_time, strlen(elapsed_time));
				while(e_uart2_sending());
				*/
				//for(r=0;r<30000;r++);
				/*
				memset(elapsed_time, 0x0, 20);
				sprintf(elapsed_time,"[%d] Ticks=%lu\n", i, timer_ticks);
				e_send_uart2_char(elapsed_time, strlen(elapsed_time));
				while(e_uart2_sending());
				*/
			//}
			//c1 = clock();
			//t1 = time(NULL);
			//T1CONbits.TON = 0;		// start Timer1
	
			/*
			memset(elapsed_time, 0x0, 40);
			sprintf(elapsed_time,"FCY=%.3f, PR=%u, ticks=%lu\r\n", FCY, PR1, timer_ticks);
			e_send_uart2_char(elapsed_time, strlen(elapsed_time));
			while(e_uart2_sending());
			memset(elapsed_time, 0x0, 40);	
			//timing in seconds
			timing = (double)(1.0/(FCY/256.0))*(double)TMR1 + (double)(1.0/(FCY/256.0))*(double)PR1*(double)timer_ticks;
			unsigned int timer1 = TMR1;
			sprintf(elapsed_time, "TMR1=%u,time=%f s\r\n", timer1, timing);	
			e_send_uart2_char(elapsed_time, strlen(elapsed_time));
			while(e_uart2_sending());
			memset(elapsed_time, 0x0, 40);	
			*/
			sprintf(elapsed_time, "%d) sent=%d, kbps=%f\r\n", series, bytes_sent, (double)(bytes_sent*8.0)/(1000.0*timing));
			series++;
			e_send_uart2_char(elapsed_time, strlen(elapsed_time));
			while(e_uart2_sending());		
		
		}
		/*
		memset(elapsed_time, 0x0, 20);
		sprintf(elapsed_time,"%lu,%lu,%lu,%.3f,%.3f,%.3f,%.3f s\n", c0, c1, CLOCKS_PER_SEC, (double)c0, (double)c1, (double)CLOCKS_PER_SEC, (double)((c1 - c0)/CLOCKS_PER_SEC) );
		//sprintf(elapsed_time, "Time (CPU): %f s\n", ( (double)(c1 - c0) / ( (double)CLOCKS_PER_SEC)));
		e_send_uart2_char(elapsed_time, strlen(elapsed_time));
		while(e_uart2_sending());	
		memset(elapsed_time, 0x0, 20);	
		sprintf(elapsed_time, "Time: %d s\n", (t1-t0));
		e_send_uart2_char(elapsed_time, strlen(elapsed_time));
		while(e_uart2_sending());
		*/

		
		e_bt_release_SPP_link();
	
	} else if (selector==10) {
		//the following 3 lines are needed to release completely the camera on the robot and let use the camera on the gumstix ext.
		SIO_C_DIR= INPUT_PIN;

		SIO_D_DIR= INPUT_PIN;
		CAM_RESET_DIR = INPUT_PIN;


		ADCON1bits.ADON = 0;	//Stop A/D
		T1CONbits.TON = 0;	//Stop timer1
		T2CONbits.TON = 0;	//Stop timer2

		T3CONbits.TON = 0;	//Stop timer3
		U1MODEbits.UARTEN = 0;	//Disable uart1
		U2MODEbits.UARTEN = 0;	//Disable uart2
		I2CCONbits.I2CEN=0;	//Disable I2C

		IEC0bits.ADIE = 0;	//Disable A/D interrupt
		STOP_TMR1;		//Disable timer1 interrupt
		STOP_TMR2;		//Disable timer2 interrupt
		STOP_TMR3;		//Disable timer3 interrupt
		IEC0bits.U1RXIE = 0;	//Disable uart1 rx interrupt
		IEC0bits.U1TXIE = 0;	//Disable uart1 tx interrupt
		IEC1bits.U2RXIE = 0;	//Disable uart2 rx interrupt
		IEC1bits.U2TXIE = 0;	//Disable uart2 tx interrupt
		e_i2c_disable();	//Disable I2C interrupt

/*
		IEC0bits.INT0IE=0;

        	IEC0bits.IC1IE=0;

        	IEC0bits.OC1IE=0;

        	IEC0bits.T1IE=0;

        	IEC0bits.IC2IE=0;

        	IEC0bits.OC2IE=0;

        	IEC0bits.T2IE=0;

        	IEC0bits.T3IE=0;

        	IEC0bits.SPI1IE=0;

        	IEC0bits.U1RXIE=0;

        	IEC0bits.U1TXIE=0;

        	IEC0bits.ADIE=0;

        	IEC0bits.NVMIE=0;

        	IEC0bits.SI2CIE=0;

        	IEC0bits.MI2CIE=0;

        	IEC0bits.CNIE=0;



        	IEC1bits.INT1IE=0;

        	IEC1bits.IC7IE=0;

        	IEC1bits.IC8IE=0;

        	IEC1bits.OC3IE=0;

        	IEC1bits.OC4IE=0;

        	IEC1bits.T4IE=0;

        	IEC1bits.T5IE=0;

        	IEC1bits.INT2IE=0;

        	IEC1bits.U2RXIE=0;

        	IEC1bits.U2TXIE=0;

        	IEC1bits.SPI2IE=0;

        	IEC1bits.C1IE=0;

        	IEC1bits.IC3IE=0;

        	IEC1bits.IC4IE=0;

        	IEC1bits.IC5IE=0;

        	IEC1bits.IC6IE=0;



        	IEC2bits.OC5IE=0;

        	IEC2bits.OC6IE=0;

        	IEC2bits.OC7IE=0;

        	IEC2bits.OC8IE=0;

        	IEC2bits.INT3IE=0;

        	IEC2bits.INT4IE=0;

        	IEC2bits.C2IE=0;

        	//IEC2bits.PWMIE=0;

        	//IEC2bits.QEIIE=0;

        	//IEC2bits.FLTAIE=0;

        	//IEC2bits.FLTBIE=0;

*/
		SLEEP();	//sleep mode

	} else if (selector==11) {	// BT configuration	

		 sprintf(buffer, "\f\a\n\n\rWELCOME to the protocol on e-Puck Bluetooth calibration");

		 e_send_uart2_char(buffer, strlen(buffer));  //send string

		while(e_uart2_sending());

		memset(buffer, 0x0, 120); 
		sprintf(buffer, "\n\rPress H (return) for help");

		 e_send_uart2_char(buffer, strlen(buffer));  //send string

 		while(e_uart2_sending());

		memset(buffer, 0x0, 120);
		
		while(1)

		{	

		   i = 0;

			c=0;

		   do

		   {

		     	if (e_getchar_uart2(&buffer[100+i])) //&command[i]))

				{	

					c=buffer[100+i];//command[i];

		    		i++;

				}	

		   }

		   while (((char)c != '\n')&&((char)c != '\x0d'));

		   buffer[100+i]='\0'; //command[i]='\0';

			

		memset(buffer, 0x0, 100);

		   switch (buffer[100]) 

		   { 

			case 'P':	e_bt_read_local_pin_number(buffer);

						sprintf(&buffer[50],"\n\rPIN code = %s",buffer);

						break;

			case 'O':	sscanf(&buffer[100],"O,%s\n",buffer);

						if(e_bt_write_local_pin_number(buffer))

							sprintf(&buffer[50],"\n\rError writting PIN");

						else

							sprintf(&buffer[50],"\n\rPIN code = %s",buffer);

						break;

			case 'M':	sscanf(&buffer[100],"M,%s\n",buffer);

						if(e_bt_write_local_name(buffer))

							sprintf(&buffer[50],"\n\rError writting Name");

						else

							sprintf(&buffer[50],"\n\rFriendly name = %s",buffer);

						break;

			case 'S':	sscanf(&buffer[100],"S,%s\n",buffer);

						if(e_bt_write_local_pin_number(buffer))

							sprintf(&buffer[50],"\n\rError writting PIN");

						else

							sprintf(&buffer[50],"\n\rPIN code = %s",buffer);

						e_send_uart2_char(&buffer[50],strlen(&buffer[50])); 

						while(e_uart2_sending());

						sprintf(&buffer[100],"e-puck_%s",buffer);

						if(e_bt_write_local_name(&buffer[100]))

							sprintf(&buffer[50],"\n\rError writting Name");

						else

							sprintf(&buffer[50],"\n\rFriendly name = %s",&buffer[100]);

						break;

			case 'N':	e_bt_read_local_name(buffer);

						sprintf(&buffer[50],"\n\rFriendly name = %s",buffer);

						break;

			case 'R':	version=e_bt_reset();

						sprintf(&buffer[50],"\n\rReset ok Firmware = %d",version);

						break;

			case 'H':	uart2_send_static_text("\n\r \"M,Name\" Write Name for Friendly Bluetooth name");

						uart2_send_static_text("\n\r \"N\" Read actual Friendly Bluetooth name");

						uart2_send_static_text("\n\r \"O,#\" Write # PIN number");

						uart2_send_static_text("\n\r \"P\" Read actual PIN number"); 

						uart2_send_static_text("\n\r \"R\" Soft reset Bluetooth module");	

						uart2_send_static_text("\n\r \"S,#\" Write # PIN number and same time e-puck_#");	

						

						buffer[50]='\n';

						buffer[51]='\0';

						break;

		   	default:    sprintf(&buffer[50],"\n\rz,Command not found");

		                 	break;

		   }

		   e_send_uart2_char(&buffer[50],strlen(&buffer[50]));

		   while(e_uart2_sending());

		 }		

	} else if (selector==12) {	// Global test
		sprintf(buffer, "%c%c%d;%d;%dm", 0x1B, 0x5B, 1, 37, 44);	//bold
		e_send_uart2_char(buffer, strlen(buffer));

		// calibrate proximity sensors
		uart2_send_static_text("\f\aStarting calibration - Remove any object in sensors range\r\n");
		e_calibrate_ir();
		uart2_send_static_text("Calibration finished\r\n");

		// sound
		e_init_sound();
		e_play_sound(11028,8016);
		wait(800000);

		// set motor speed
		e_set_speed_left(100);
		e_set_speed_right(100);

		// init camera
		e_poxxxx_init_cam();
		e_poxxxx_config_cam((ARRAY_WIDTH -40*8)/2,(ARRAY_HEIGHT-40*8)/2,40*8,40*8,8,8,RGB_565_MODE);
		e_poxxxx_set_mirror(1,1);
		e_poxxxx_write_cam_registers();

		// set body led
		e_set_body_led(1);
	
		// set front led
		e_set_front_led(1);
	
		// set others LEDs
		e_set_led(8,1);

		uart2_send_static_text("\f\a");
		while(1) {

			// read accelerometer
			//ESC]2k => clear line
			sprintf(buffer,"%c%cH\r\nACCELEROMETER\r\n%c%c2K%4d, %4d, %4d\r\n\n", 0x1B, 0x5B, 0x1B, 0x5B, e_get_acc(0),e_get_acc(1),e_get_acc(2));
			e_send_uart2_char(buffer, strlen(buffer));
			while(e_uart2_sending());

			// read selector position
			selector = SELECTOR0 + 2*SELECTOR1 + 4*SELECTOR2 + 8*SELECTOR3;
			sprintf(buffer,"SELECTOR\r\n%c%c2K%2d\r\n\n",0x1B, 0x5B, selector);
			e_send_uart2_char(buffer, strlen(buffer));
			while(e_uart2_sending());

			// read IR
                  	sprintf(buffer,"IR\r\n%c%c2Kcheck : 0x%x, address : 0x%x, data : 0x%x\r\n\n", 0x1B, 0x5B, e_get_check(), e_get_address(), e_get_data());
                  	e_send_uart2_char(buffer, strlen(buffer));
			while(e_uart2_sending());

			// read proximity sensors
			sprintf(buffer,"PROXIMITY SENSORS\r\n%c%c2K%4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d\r\n\n", 0x1B, 0x5B, 
			e_get_prox(0),e_get_prox(1),e_get_prox(2),e_get_prox(3),
			e_get_prox(4),e_get_prox(5),e_get_prox(6),e_get_prox(7));
			e_send_uart2_char(buffer, strlen(buffer));
			while(e_uart2_sending());

			sprintf(buffer,"AMBIENT LIGHT SENSORS\r\n%c%c2K%4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d\r\n\n", 0x1B, 0x5B,
			e_get_ambient_light(0),e_get_ambient_light(1),e_get_ambient_light(2),e_get_ambient_light(3),
			e_get_ambient_light(4),e_get_ambient_light(5),e_get_ambient_light(6),e_get_ambient_light(7));
			e_send_uart2_char(buffer, strlen(buffer));
			while(e_uart2_sending());

			// micro
			sprintf(buffer,"MICROPHONE\r\n%c%c2K%4d,%4d,%4d\r\n\n",0x1B, 0x5B, e_get_micro_volume(0),e_get_micro_volume(1),e_get_micro_volume(2));
			e_send_uart2_char(buffer, strlen(buffer));
			while(e_uart2_sending());

			//camera
			e_poxxxx_launch_capture(&buffer[0]);
			while(!e_poxxxx_is_img_ready());
                        r = (int)buffer[0]&0xF8;
                        g = (int)(buffer[0]&0x07)<<5 | (buffer[1]&0xE0)>>3;
                        b = (int)(buffer[1]&0x1F)<<3;
			sprintf(&buffer[3200], "CAMERA\r\n%c%c2KR=%3d, G=%3d, B=%3d\r\n\n",0x1B, 0x5B, r, g, b);
			e_send_uart2_char(&buffer[3200], strlen(&buffer[3200]));
			while(e_uart2_sending());
			
			for(i=0; i<100; i++);
		} 
	} else if (selector==13) {	// uart 1 to uart 2 transponder
		//e_init_uart2();// 230400   // initialize UART to 115200 Kbaud
		e_send_uart2_char(buffer, strlen(buffer));
		while (1) {
			if(e_getchar_uart1(&c)) e_send_uart2_char(&c, 1);
			if(e_getchar_uart2(&c)) e_send_uart1_char(&c, 1);
		}
	} else if (selector==14) {
		run_breitenberg_follower();
	} else if (selector==15) {	// simple dust cleaner behaviour
		run_DustCleaner();
	} else {
		run_breitenberg_shocker();
	}

	while(1);
	return 0;
}

